# -*- coding: utf-8 -*-
"""
本模块存放工具函数
"""
import base64
import aiofiles

JSON_ENCODING = "utf-8"


def __init():
    pass


def add_options_and_position(params, options=None, position=None):
    """
    Convenience method for adding options and position to parameters.
    """
    if options:
        params.append(options)
    if position:
        if not isinstance(position, int):
            try:
                position = int(position)
            except ValueError:
                position = -1
        if position >= 0:
            params.append(position)
    return params


async def b64encode_file(path):
    """
    读取文件，转换b64编码
    """
    async with aiofiles.open(path, 'rb') as handle:
        return str(base64.b64encode(await handle.read()), JSON_ENCODING)


def get_status(response):
    """
    Process a status response.
    """
    if response:
        try:
            return response['status']
        except KeyError:
            return 'error'
    else:
        return 'error'


def read_configfile(path: str, prex: str = "--"):
    """
    从配置文件中读取可用配置
    :param path: aria2配置文件路径
    :param prex: yield之前的前缀
    :return:
    """
    with open(path, "r") as f:
        for line in f.readlines():
            line = line.strip()
            if line and not line.startswith("#"):
                temp = prex + line
                yield temp


if __name__ == "__main__":
    # import asyncio
    #
    # loop = asyncio.get_event_loop()
    # task = loop.create_task(b64encode_file(
    #     "F:\jhc\PanDownload\[kisssub.org][Moozzi2] 国家队比翼之吻 Darling in the Franxx 1-24 (BD 1920x1080 x.264 FLACx2).torrent"))
    # loop.run_until_complete(task)
    # print(task.result())
    # with open("F:\jhc\PanDownload\[kisssub.org][Moozzi2] 国家队比翼之吻 Darling in the Franxx 1-24 (BD 1920x1080 x.264 FLACx2).torrent","rb") as f:
    #     a=f.read()
    #     b=base64.b64encode(a)
    #     c=b.encode("utf-8")
    #     pass
    pass



# -*- coding: utf-8 -*-
"""
本模块负责管理aria2进程
"""

import os
import subprocess
from typing import Dict
from functools import wraps
import asyncio

# --------------------------#
ENCODING = "gbk"

# --------------------------#

cache: Dict[str, object] = {}


def single_instance(cls: type):
    """
    单例模式装饰器，如果两个aria2进程一同开启必定端口冲突
    :param cls: 要装饰的类
    :return:
    """

    @wraps(cls)
    def inner(*args, **kw):
        class_name = cls.__name__
        if class_name in cache:
            return cache[class_name]
        else:
            instance = cls(*args, **kw)
            cache[class_name] = instance
            return instance
    return inner


@single_instance
class Aria2Server:
    """
    aria2进程对象
    """

    def __init__(self, *args: str, daemon=False):
        """
        :param args: 启动aria2的命令行参数
        :param daemon: True:aria2随python解释器同生共死
        """
        if args:
            self.cmd = list(args)
        else:
            self.cmd = []
        if daemon:
            self.cmd.append('--stop-with-process={:d}'.format(os.getpid()))
        self.process = None
        self._is_running = False

    def start(self):
        self.process = subprocess.Popen(self.cmd)
        self._is_running = True

    def wait(self):
        """
        等待进程结束
        :return:
        """
        self.process.wait()
        self._is_running = False

    def terminate(self):
        self.process.terminate()
        self.wait()

    def kill(self):
        self.process.kill()
        self.wait()

    @property
    def pid(self):
        return self.process.pid

    @property
    def returncode(self):
        return self.process.returncode

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._is_running:
            self.terminate()


@single_instance
class AsyncAria2Server(Aria2Server):
    """
    aria2进程对象
    异步io
    """

    def __init__(self, *args: str, daemon=False):
        super().__init__(*args, daemon=daemon)

    async def start(self):
        cmd = ""
        for i in self.cmd:
            cmd += i + " "
        # program, *args = self.cmd
        self.process = await asyncio.create_subprocess_shell(cmd)
        self._is_running = True

    async def wait(self):
        await self.process.wait()
        self._is_running = False

    async def terminate(self):
        self.process.terminate()
        await self.wait()

    async def kill(self):
        self.process.kill()
        await self.wait()

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self._is_running:
            await self.terminate()


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-
import aioaria2
import asyncio
from pprint import pprint

q = asyncio.Queue()

failed_q = asyncio.Queue()

HOST = "http://127.0.0.1:6800/jsonrpc"


def callback(future):
    print("下载开始{0}".format(future.result()))


def callback2(future):
    print("下载完成{0}".format(future.result()))


def callback3(future):
    data = future.result()
    print("下载中断{0}".format(data))


def onresult(future):
    print("结果")
    pprint(future.result())


async def get_client():
    async with aioaria2.Aria2HttpClient("id", HOST, "normal",
                                        token="admin", queue=q) as client:
        pprint(await client.addUri(["http://odrive.aptx.xin/%E5%8A%A8%E7%94%BB/2004/200445.zip"]))
        # pprint(await client.getFiles("6b25a38d701dee4c"))

    pass


async def get_trigger():
    client = await aioaria2.Aria2WebsocketTrigger.create("id",
                                                         HOST,
                                                         token="admin",
                                                         queue=q)

    client.onDownloadStart(callback)
    client.onDownloadComplete(callback2)
    client.onDownloadError(callback3)
    client.onResullt(onresult)
    await client.start_ws()
    pass


def main():
    loop = asyncio.get_event_loop()
    # loop.call_soon_threadsafe()
    asyncio.run_coroutine_threadsafe()
    tasks = [loop.create_task(get_trigger()), loop.create_task(get_client())]
    loop.run_until_complete(asyncio.wait(tasks))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os
import re


def get_version() -> str:
    path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "aioaria2", "__init__.py")
    with open(path, "r", encoding="utf-8") as f:
        data = f.read()
    result = re.findall(r"(?<=__version__ = \")\S+(?=\")", data)
    return result[0]


def get_dis():
    with open("README.md", "r", encoding="utf-8") as f:
        return f.read()


def main():
    version: str = get_version()

    dis = get_dis()
    setup(
        name="aioaria2",
        version=version,
        packages=find_packages(),
        keywords=["asyncio", "Aria2"],
        description="Support Aria2 rpc client and manage server with async/await",
        long_description_content_type="text/markdown",
        long_description=dis,
        author=["帝国皇家近卫军", "synodriver"],
        author_email="diguohuangjiajinweijun@gmail.com",
        maintainer="v-vinson",
        python_requires=">=3.6",
        install_requires=["aiohttp", "aiofiles"],
        license='GPLv3',
        classifilers=[
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: Implementation :: CPython"
        ],
        include_package_data=True
    )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import asyncio


async def my_callback(result):
    print("my_callback got:", result)
    return "My return value is ignored"


async def coro(number):
    await asyncio.sleep(number)
    return number + 1


async def add_success_callback(fut, callback):
    result = await fut
    await callback(result)
    return result


# loop = asyncio.get_event_loop()
# task = asyncio.ensure_future(coro(1))
task = add_success_callback(coro(1), my_callback)
response = asyncio.run(task)
print("response:", response)
